#define BACKGROUND_C

#include "gba.h"
#include "backgrounds.h"
#include "screenmode.h"
#include "dma.h"



extern FIXED COS[360];
extern FIXED SIN[360];



Mode7Array Mode7LUT[360/8] IN_EWRAM;


void CalculateMode7LUT(void)
{
    int count, angle; //some looping variables
    
    int center_x = 80;
    int center_y = 160;
    FIXED zoom = (1<<8)+80;  //set up zoom for no zoom.
    
    
    
    for(angle = 0; angle < 360; angle += 8)
    {
        for(count = 0; count < 160; count++)
        {
            center_y = (center_y * zoom)>>8;
            center_x = (center_x * zoom)>>8;
            Mode7LUT[angle>>3].RegisterValues[count].DX = (-center_y*SIN[angle]-center_x*COS[angle]);
            Mode7LUT[angle>>3].RegisterValues[count].DY = (-center_y*COS[angle]+center_x*SIN[angle]);
            Mode7LUT[angle>>3].RegisterValues[count].PA = (COS[angle]*zoom)>>8;
            Mode7LUT[angle>>3].RegisterValues[count].PB = (SIN[angle]*zoom)>>8;
            Mode7LUT[angle>>3].RegisterValues[count].PC = (-SIN[angle]*zoom)>>8;
            Mode7LUT[angle>>3].RegisterValues[count].PD = (COS[angle]*zoom)>>8;
            zoom -= 1;  //change this value to change the apperant angle.
        }
        zoom = (1<<8)+80;   
    }   
}

void DoMode7(int angle)
{
    DMA_Copy(0, (void*)&Mode7LUT[angle], (void*) &REG_BG2PA, 4, DMA_ENABLE 
            | DMA_TIMEING_HBLANK | DMA_DEST_RELOAD | DMA_REPEATE | DMA_16);
}

void EnableBackground(Bg* bg)
{
    u16 temp;

    bg->tileData = (u16*)CharBaseBlock(bg->charBaseBlock);
    bg->mapData = (u16*)ScreenBaseBlock(bg->screenBaseBlock);
    temp = bg->size | (bg->charBaseBlock<<CHAR_SHIFT) | (bg->screenBaseBlock<<SCREEN_SHIFT) 
        | bg->colorMode | bg->mosaic | (bg->wraparound<<13);

    bg->PA = 256;  // horizontal texture step in x
    bg->PB = 0;    // vertical texture step in x
    bg->PC = 0;    // horizontal texture step in y
    bg->PD = 256;  // vertical texture step in y

    switch(bg->number)
    {
    case 0:
        {
            REG_BG0CNT = temp;
            REG_DISPCNT |= BG0_ENABLE;
        }break;
    case 1:
        {
            REG_BG1CNT = temp;
            REG_DISPCNT |= BG1_ENABLE;
        }break;
    case 2:
        {
            REG_BG2CNT = temp;
            REG_DISPCNT |= BG2_ENABLE;
        }break;
    case 3:
        {
            REG_BG3CNT = temp;
            REG_DISPCNT |= BG3_ENABLE;

            REG_BG3PA = 256;  // horizontal texture step in x
            REG_BG3PB = 0;    // vertical texture step in x
            REG_BG3PC = 0;    // horizontal texture step in y
            REG_BG3PD = 128;  // vertical texture step in y
        }break;
    
    default:break;
    
    }
}

void UpdateBackground(Bg* bg)
{
    switch(bg->number)
    {
    case 0:
        REG_BG0HOFS = bg->x_scroll;
        REG_BG0VOFS = bg->y_scroll;
        break;
    case 1:
        REG_BG1HOFS = bg->x_scroll;
        REG_BG1VOFS = bg->y_scroll;
        break;
    case 2:
        if(!(REG_DISPCNT & MODE_0))//it is a rot background
        {
            REG_BG2X = bg->DX;
            REG_BG2Y = bg->DY;
            
            REG_BG2PA = bg->PA;
            REG_BG2PB = bg->PB;
            REG_BG2PC = bg->PC;
            REG_BG2PD = bg->PD;
        }
        else  //it is a text background
        {
            REG_BG2HOFS = bg->x_scroll;
            REG_BG2VOFS = bg->y_scroll;
        }
        break;
    case 3:
        if(!(REG_DISPCNT & MODE_0))//it is a rot background
        {
            REG_BG3X = bg->DX;
            REG_BG3Y = bg->DY;
            
            REG_BG3PA = bg->PA;
            REG_BG3PB = bg->PB;
            REG_BG3PC = bg->PC;
            REG_BG3PD = bg->PD;
        }
        else //it is a text background
        {
            REG_BG3HOFS = bg->x_scroll;
            REG_BG3VOFS = bg->y_scroll;
        }
        break;
    default: break;
    }
}

void RotateBackground(Bg* bg, int angle,int center_x, int center_y, FIXED zoom)
{
    
    center_y = (center_y * zoom)>>8;
    center_x = (center_x * zoom)>>8;
    
    bg->DX = (bg->x_scroll<<8)+((-center_y*SIN[angle]-center_x*COS[angle])>>8);
    bg->DY = (bg->y_scroll<<8)+((-center_y*COS[angle]+center_x*SIN[angle])>>8);
    
    bg->PA = (COS[angle]*zoom)>>16;  //cos&sin are LUTs that are .8 fixed numbers
    bg->PB = (SIN[angle]*zoom)>>16;  //zoom is also fixed
    bg->PC = (-SIN[angle]*zoom)>>16;
    bg->PD = (COS[angle]*zoom)>>16;
}
